(*This file does the 3 positive dominance tests for Lemma B3522 in the monograph*)
(*This is the main step in showing that the SECOND SYMMETRIZATION is energy deceasing*)
(*in the given domain. The second symmetrization is what we call vertical symmetrization*)



(*******INTERACTION ENERGY***********************************)
(*stereographic projection*)
ST[{x_,y_,z_}]:={x/(1-z),y/(1-z)}
  
  (*Inverse stereographic projection*)
IST[{x_,y_}]:={2 x/(1+x x + y y),2 y/(1+x x + y y),1-2/(1+x x + y y)}

  (*square distance*)
DIST2[v_,w_]:=(v-w).(v-w);

(*1/r^2 potential energy*)
ENERGY2[p_,q_]:=1/DIST2[IST[p],IST[q]];
(***********************************************************)j


(******************CREATES THE POLYNOMIALS*******************)
(*This is the parameterization*)
f01[t_]:=(416 (1-t) + 498 t)/512;
f02[t_]:=(+16 t)/512
f1[t_]:=(348 (1-t) + 465 t)/512;
(*note the minus sign*)
g0[t_]:=(-59 t)/512 

(*This is the basic energy function*)
Module[{point0,point1,point3,a1,a3},
       A[a_,b_,c_,d_]:=(point3={0,+f1[a]+g0[c]};
                       point0={f01[b],f02[d]};
                       point1={0,-f1[a]+g0[c]};
		       a1=ENERGY2[point0,point1];
		       a3=ENERGY2[point0,point3];
			Power[a1,6]+Power[a3,6])];

(*This computes the energy difference*)
  B[a_,b_,c_,d_]:=A[a,b,c,d] - A[a,b,0,0];

(*creates the polynomial, the coefficient list, the monomial list*)
poly2=Expand[Numerator[Together[Expand[B[a,b,c,d]]]]]



  
(**********5 VARIABLE POSDOM TEST*********************************)
  (*We only need the 3 variable and 4 variable case here but the extra variables do not matter*)
  

Module[{TRANS5,MAX1,MAX2,MAX3,MAX4,MAX5,coeffs,monomials,exponents,LIST},Posdom5[poly_]:=(
		 coeffs=Table[poly[[j]]/.{a->1,b->1,c->1,d->1,e->1},{j,1,Length[poly]}];
                 monomials=Table[poly[[j]]/coeffs[[j]],{j,1,Length[poly]}];
                 exponents=Table[ExtractExponent5[monomials[[j]]],{j,1,Length[poly]}];

                 (*getting the maximum exponents*)
                 TRANS5=Transpose[exponents];
                 MAX1=Max[TRANS5[[1]]];
                 MAX2=Max[TRANS5[[2]]];
                 MAX3=Max[TRANS5[[3]]];
                 MAX4=Max[TRANS5[[4]]];
                 MAX5=Max[TRANS5[[5]]];
Print["      exponent limits ",MAX1," ",MAX2," ",MAX3," ",MAX4," ",MAX5]; 
                 (*done getting maximum exponents*)

		 (*tailored to the one polynomial we use it on*)
LIST=Table[Table[Table[Table[Table[SumCoeffs[{i1,i2,i3,i4,i5},exponents,coeffs],{i1,0,MAX1}],{i2,0,MAX2}],{i3,0,MAX3}],{i4,0,MAX4}],{i5,0,MAX5}];
                 {Min[Flatten[LIST]],Max[Flatten[LIST]]})];



                 (*supporting routines*)

SumCoeffs[BOUND_,exponents_,coeffs_]:=Sum[SingleTerm[BOUND,exponents,coeffs,k],{k,1,Length[coeffs]}]
SingleTerm[BOUND_,exponents_,coeffs_,k_]:=ExponentDominated[BOUND,exponents[[k]]] coeffs[[k]];
ExponentDominated[BOUND_,small_]:=If[Min[BOUND-small]<0,0,1];
                 

(*extracts the exponent.*)

Module[{val,fac,P},ExtractExponent5[mono_]:=(
		  P=Table[Prime[j],{j,100,105}];
                  val=mono/.{a->P[[1]],b->P[[2]],c->P[[3]],d->P[[4]],e->P[[5]]};
                  fac=FactorInteger[val];
                  Table[ex5[fac,P[[j]]],{j,1,5}])];

Module[{pos},ex5[fac_,p_]:=(
              pos=Position[fac,p];
              If[Length[pos]==0,0,fac[[pos[[1,1]],2]]])];
            
(**************END POSDOM5  METHOD***********************************)

       

Module[{coeffs,monomials,exponents},TheWorks[poly_]:=(
                 coeffs=Table[poly[[j,1]],{j,1,Length[poly]}];
                 monomials=Table[poly[[j]]/coeffs[[j]],{j,1,Length[poly]}];
                 exponents=Table[ExtractExponent5[monomials[[j]]],{j,1,Length[poly]}];
		 {coeffs,exponents,monomials})];


Print["done making polynomial"];

works=TheWorks[poly2];
coef=works[[1]];
expo=works[[2]];
mono=works[[3]];


(**************************MASSAGING THE POLYNOMIAL***********************)

max=Max[coef];
coef2=Table[Floor[Power[10,10] coef[[j]]/max],{j,1,Length[coef]}]


(**Now we count the number of terms with -1 coefficient*)
MINUS=Flatten[Position[coef2,-1]];
NUMBER=Length[MINUS];
Print["number of -1 terms: ",NUMBER];

(*fifth variable is 0*)

MINUSLIST=Table[expo[[MINUS[[j]]]],{j,1,Length[MINUS]}];
(*Check the every such term is divisible by one of C^2, D^2, or CD*)
CHECK=Min[Table[MINUSLIST[[j]].{0,0,1,1,0},{j,1,Length[MINUSLIST]}]];
Print["      divisibility check (value should be 2): ",CHECK];


kill[x_]:=If[x==-1,0,x];

POSCC=Position[expo,{0,0,0,2,0}];
POSDD=Position[expo,{0,0,2,0,0}];
POSCD=Position[expo,{0,0,1,1,0}];
POSCC=POSCC[[1,1]];
POSDD=POSDD[[1,1]];
POSCD=POSCD[[1,1]];

coef3=coef2;
coef3[[POSCC]]=coef3[[POSCC]]-NUMBER;
coef3[[POSDD]]=coef3[[POSDD]]-NUMBER;
coef3[[POSCD]]= coef3[[POSCD]]-NUMBER;

coef3=Table[kill[coef3[[j]]],{j,1,Length[coef3]}]

PSI=Expand[coef3.mono]

(************************************************************)


(***************THE TESTS**********************************)
Print["           posdom tests"];

  PSIX=Expand[PSI//.a->0];
  PSIaX=D[PSI,a]//.a->0;
  PSIaaX=D[PSI,{a,2}]//.a->0;


Print["    3 tests for D[PSI,{a,2}] restricted to a=0: "];
  Print[Posdom5[Expand[4 D[PSIaaX,c] +  D[PSIaaX,d]]]];
  Print[Posdom5[Expand[PSIaaX//.c->0]]];
  Print[Posdom5[Expand[PSIaaX//.d->0]]];

Print["    3 tests for D[PSI,{a,1}] restricted to a=0: "];
  Print[Posdom5[Expand[4  D[PSIaX,c] +  D[PSIaX,d]]]];
  Print[Posdom5[Expand[PSIaX//.c->0]]];
  Print[Posdom5[Expand[PSIaX//.d->0]]];


Print["    3 tests for PSI restricted to a=0:  (first one fails)"];
  Print[Posdom5[Expand[4  D[PSIX,c] +   D[PSIX,d]]]];
  Print[Posdom5[Expand[PSIX//.c->0]]];
  Print[Posdom5[Expand[PSIX//.d->0]]];

  G=4 D[PSIX,c] + D[PSIX,d];
  H=G//.d->0;
  HH=H//.{b->bb}
  S0H=HH//.{bb->b/2};
  S1H=HH//.{bb->1/2  +  b/2};

Print["    tests for D[G,d] and the 2 subdivisions of G restricted to d=0"];		       
   Print[Posdom5[Expand[D[G,d]]]];	
   Print[Posdom5[Expand[S0H]]];		
   Print[Posdom5[Expand[S1H]]];	

Print[    "and here is the massive test of D[PSI,{a,3}] that takes a long time"];

Print[Posdom5[Expand[D[PSI,{a,3}]]]];






